import { app } from "./app";
import { APIException, NetworkConnectionException, ReactLifecycleException, RuntimeException } from "./Exception";
import { loggerContext } from "./platform/logger-context";
import { serializeError } from "./util/error-util";
export class LoggerImpl {
    constructor() {
        this.environmentContext = {};
        this.logQueue = [];
        this.environmentContext = loggerContext;
    }
    addContext(context) {
        this.environmentContext = { ...this.environmentContext, ...context };
    }
    info(action, info, elapsedTime) {
        return this.appendLog("OK", { action, info, elapsedTime: elapsedTime || 0 });
    }
    warn(data) {
        return this.appendLog("WARN", data);
    }
    error(data) {
        return this.appendLog("ERROR", data);
    }
    exception(exception, action, extraInfo) {
        if (exception instanceof NetworkConnectionException) {
            const info = {
                ...extraInfo,
                url: exception.requestURL,
                errorObject: serializeError(exception.errorObject),
            };
            return this.appendLog("WARN", { action, errorCode: "NETWORK_FAILURE", errorMessage: exception.message, info, elapsedTime: 0 });
        }
        else {
            const info = { ...extraInfo };
            let isWarning = false;
            let errorCode = "OTHER_ERROR";
            if (exception instanceof APIException) {
                errorCode = `API_ERROR_${exception.statusCode}`;
                // Following cases are treated as expected
                if ([401, 403, 426, 503].includes(exception.statusCode)) {
                    isWarning = true;
                }
                else if (exception.statusCode === 400) {
                    if (exception.errorCode === "VALIDATION_ERROR") {
                        errorCode = "API_VALIDATION_ERROR";
                    }
                    else {
                        isWarning = true;
                    }
                }
                info.requestURL = exception.requestURL;
                if (exception.errorCode) {
                    info.errorCode = exception.errorCode;
                }
                if (exception.errorId) {
                    info.errorId = exception.errorId;
                }
            }
            else if (exception instanceof ReactLifecycleException) {
                errorCode = "LIFECYCLE_ERROR";
                info.stackTrace = exception.componentStack;
                info.appState = JSON.stringify(app.store.getState().app);
            }
            else if (exception instanceof RuntimeException) {
                errorCode = "RUNTIME_ERROR";
                info.errorObject = serializeError(exception.errorObject);
                info.appState = JSON.stringify(app.store.getState().app);
            }
            return this.appendLog(isWarning ? "WARN" : "ERROR", { action, errorCode, errorMessage: exception.message, info, elapsedTime: 0 });
        }
    }
    collect() {
        return this.logQueue;
    }
    empty() {
        this.logQueue = [];
    }
    appendLog(result, data) {
        const completeContext = {};
        Object.entries(this.environmentContext).map(([key, value]) => {
            if (typeof value === "string") {
                completeContext[key] = value;
            }
            else {
                let evaluatedResult;
                try {
                    evaluatedResult = value();
                }
                catch (e) {
                    evaluatedResult = "[ERROR] " + serializeError(e);
                    console.warn("Fail to execute logger context: " + serializeError(e));
                }
                completeContext[key] = evaluatedResult;
            }
        });
        const event = {
            date: new Date(),
            result,
            context: completeContext,
            ...data,
        };
        this.logQueue.push(event);
    }
}
//# sourceMappingURL=Logger.js.map